@@ -1,28 +1,37 @@
 """
 Convert NetworkX graphs to and from other formats.
 
-from_whatever attemps to guess the input format
+from_whatever attempts to guess the input format
 
-Create a 10 node random digraph
+Example: Create a 10 node random digraph
 
->>> from networkx import *
 >>> import numpy
 >>> a=numpy.reshape(numpy.random.random_integers(0,1,size=100),(10,10))
->>> D=from_whatever(a,create_using=DiGraph()) # or D=DiGraph(a) 
+>>> D=nx.from_whatever(a,create_using=nx.DiGraph()) 
+
+or
+
+>>> D=nx.DiGraph(a) 
 
 For graphviz formats see networkx.drawing.nx_pygraphviz
 or networkx.drawing.nx_pydot.
 
-$Id: convert.py 701 2007-11-08 05:08:53Z aric $
 """
 __author__ = """Aric Hagberg (hagberg@lanl.gov)"""
-#    Copyright (C) 2006 by 
+#    Copyright (C) 2006-2008 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    Distributed under the terms of the GNU Lesser General Public License
 #    http://www.gnu.org/copyleft/lesser.html
 
+
+__all__ = ['from_whatever', 
+           'from_dict_of_dicts', 'to_dict_of_dicts',
+           'from_dict_of_lists', 'to_dict_of_lists',
+           'from_numpy_matrix', 'to_numpy_matrix',
+           'from_scipy_sparse_matrix', 'to_scipy_sparse_matrix']
+
 import networkx
 
 def _prep_create_using(create_using):
@@ -37,15 +46,15 @@ def _prep_create_using(create_using):
     if create_using is None:
         G=networkx.Graph()
     else:
+        G=create_using
         try:
-            G=create_using
             G.clear()
         except:
             raise TypeError("Input graph is not a networkx graph type")
     return G
 
 
-def from_whatever(thing,create_using=None):
+def from_whatever(thing,create_using=None,multigraph_input=False):
     """Attempt to make a NetworkX graph from an known type.
 
     Current known types are:
@@ -58,7 +67,24 @@ def from_whatever(thing,create_using=None):
        scipy sparse matrix
        pygraphviz agraph
 
+    If multigraph_input is True and thing is a dict_of_dicts,
+    try to create a multigraph assuming dict_of_dict_of_lists.
+    If thing and create_using are both multigraphs then create
+    a multigraph from a multigraph.
+
     """
+    # NX graph
+    if hasattr(thing,"adj"):
+        try:
+            result= from_dict_of_dicts(thing.adj,\
+                    create_using=create_using,\
+                    multigraph_input=thing.multigraph)
+            result.weighted=thing.weighted
+            return result
+        except:
+            raise networkx.NetworkXError,\
+                  "Input is not a correct NetworkX graph."
+
     # pygraphviz  agraph
     if hasattr(thing,"is_strict"):
         try:
@@ -67,18 +93,11 @@ def from_whatever(thing,create_using=None):
             raise networkx.NetworkXError,\
                   "Input is not a correct pygraphviz graph."
 
-    # NX graph
-    if hasattr(thing,"add_node"):
-        try:
-            return from_dict_of_dicts(thing.adj,create_using=create_using)
-        except:
-            raise networkx.NetworkXError,\
-                  "Input is not a correct NetworkX graph."
-
     # dict of dicts/lists
     if isinstance(thing,dict):
         try:
-            return from_dict_of_dicts(thing,create_using=create_using)
+            return from_dict_of_dicts(thing,create_using=create_using,\
+                    multigraph_input=multigraph_input)
         except:
             try:
                 return from_dict_of_lists(thing,create_using=create_using)
@@ -123,7 +142,7 @@ def to_dict_of_lists(G,nodelist=None):
 
     If nodelist is defined return a dict of lists with only those nodes.
 
-    Completely ignores edge data for XGraph and XDiGraph.
+    Completely ignores edge data for MultiGraph and MultiDiGraph.
 
     """
     if nodelist is None:
@@ -141,20 +160,19 @@ def from_dict_of_lists(d,create_using=None):
     G=_prep_create_using(create_using)
     G.add_nodes_from(d)        
 
-    if hasattr(G,"allow_multiedges") and G.multiedges and not G.is_directed():
+    if G.multigraph and not G.directed:
         # a dict_of_lists can't show multiedges.  BUT for undirected graphs,
         # each edge shows up twice in the dict_of_lists.  
         # So we need to treat this case separately.
         seen={}
-        for node in d:
-            for nbr in d[node]:
-                if (node,nbr) not in seen:
+        for node,nbrlist in d.iteritems():
+            for nbr in nbrlist:
+                if nbr not in seen:
                     G.add_edge(node,nbr)
-                    seen[(nbr,node)]=1  # don't allow reverse edge to show up 
+            seen[node]=1  # don't allow reverse edge to show up 
     else:
-        for node in d:
-            for nbr in d[node]:
-                G.add_edge(node,nbr)
+        G.add_edges_from( ((node,nbr) for node,nbrlist in d.iteritems() 
+                           for nbr in nbrlist) )
     return G                         
 
 
@@ -166,88 +184,83 @@ def to_dict_of_dicts(G,nodelist=None,edge_data=None):
     If edge_data is given, the value of the dictionary will be
     set to edge_data for all edges.  This is useful to make
     an adjacency matrix type representation with 1 as the edge data.
+    If edgedata is None, the edgedata in G is used to fill the values.
+    If G is a multigraph, the edgedata is a list for each pair (u,v).
     
     """
+    dod={}
     if nodelist is None:
-        nodelist=G.nodes()
-    if edge_data is not None:
-        get_edge=lambda x,y:edge_data
-    else:
-        get_edge=G.get_edge
+        if edge_data is None:
+            for u,nbrdict in G.adjacency_iter():
+                dod[u]=nbrdict.copy()
+        else: # edge_data is not None
+            for u,nbrdict in G.adjacency_iter():
+                dod[u]=dod.fromkeys(nbrdict, edge_data)
+    else: # nodelist is not None
+        if edge_data is None:
+            for u in nodelist:
+                dod[u]={}
+                for v,data in ((v,data) for v,data in G[u].iteritems() if v in nodelist):
+                    dod[u][v]=data
+        else: # nodelist and edge_data are not None
+            for u in nodelist:
+                dod[u]={}
+                for v in ( v for v in G[u] if v in nodelist):
+                    dod[u][v]=edge_data
+    return dod
 
-    d = {}
-    for u in nodelist:
-        d[u]={}
-        for v in G.neighbors(u):
-            if v in nodelist:
-                d[u][v]=get_edge(u,v)
-    return d            
-
-
-def from_dict_of_dicts(d,create_using=None):
+def from_dict_of_dicts(d,create_using=None,multigraph_input=False):
     """Return a NetworkX graph G from a Python dictionary of dictionaries.
 
-    The value of the inner dict becomes the edge_data for the NetworkX graph
-    EVEN if create_using is a NetworkX Graph which doesn't ever use this data.
+    The value of the inner dict becomes the edge_data for the NetworkX graph.
 
-    If create_using is an XGraph/XDiGraph with multiedges==True, the edge_data
-    should be a list, though this routine does not check for that.
+    When multigraph_input is True, the values of the inner dict are assumed 
+    to be containers of edge data for multiple edges.
+    Otherwise this routine assumes the edgedata are singletons.
 
     """
     G=_prep_create_using(create_using)
     G.add_nodes_from(d)
 
-    # is this a XGraph or XDiGraph?
-    # FIXME
-    # This is a bad way to check for whether edge data exists...
-    # If someone ever creates a graph class with edge data and
-    # without an allow_multiedges method, it won't work.
-    if hasattr(G,'allow_multiedges'): # assume edge data
-        if G.multiedges:
-            # this is a NetworkX graph with multiedges=True
-            # make a copy of the list of edge data (but not the edge data)
-            if G.is_directed():
-                for u,nbrs in d.iteritems():
-                    for v,datalist in nbrs.iteritems():
-                        dl=datalist[:] # copy of the edge_data list
-                        G.pred[u][v]=dl
-                        G.succ[u][v]=dl
-            else:
-                for u,nbrs in d.iteritems():
-                    for v,datalist in nbrs.iteritems():
-                        dl=datalist[:] # copy of the edge_data list
-                        G.adj[u][v]=dl
-                        G.adj[v][u]=dl
-        else:
+    # is dict a MultiGraph or MultiDiGraph?
+    if multigraph_input:
+        # make a copy of the list of edge data (but not the edge data)
+        if G.directed:   # copy edge list
+            G.add_edges_from( ((u,v,data) for u,nbrs in d.iteritems() \
+                    for v,dl in nbrs.iteritems() for data in dl) )
+        else: # Undirected
+            seen={}   # don't add both directions of undirected graph
+            for u,nbrs in d.iteritems():
+                for v,datalist in nbrs.iteritems():
+                    if v not in seen:
+                        G.add_edges_from( ((u,v,data) for data in datalist) )
+                seen[u]=1  # don't allow reverse edge to show up 
+    else: # not a multigraph to multigraph transfer
+        if G.directed:
+            G.add_edges_from( ((u,v,data) for u,nbrs in d.iteritems() \
+                    for v,data in nbrs.iteritems()) )
+        else:    # need this if G is multigraph and slightly faster if not multigraph
+            seen={}  
             for u,nbrs in d.iteritems():
                 for v,data in nbrs.iteritems():
-                    G.add_edge(u,v,data)
-    else: # no edge data
-        for u,nbrs in d.iteritems():
-            for v in nbrs:
-                G.add_edge(u,v)
-
+                    if v not in seen:
+                        G.add_edge(u,v,data)
+                seen[u]=1
     return G                         
 
 
-
 def to_numpy_matrix(G,nodelist=None):
-    """Return adjacency matrix of graph as a numpy matrix.
+    """Return the adjacency matrix of graph G as a numpy matrix.
 
     If nodelist is defined return adjacency matrix with nodes in nodelist
-    in the order specified.  If not the ordering is whatever order
-    the method G.nodes() produces.
+    in the order specified.  All nodes must appear in nodelist or a KeyError
+    is raised.
+    If nodelist is None, the ordering is produced by G.nodes()
 
-    For Graph/DiGraph types which have no edge data 
-    The value of the entry A[u,v] is one if there is an edge u-v
-    and zero otherwise.
+    When G.weighted==False the value of the entry A[u,v] is one 
+    if there is an edge u-v and zero otherwise.
 
-    For XGraph/XDiGraph the edge data is assumed to be a weight and be
-    able to be converted to a valid numpy type (e.g. an int or a
-    float).  The value of the entry A[u,v] is the weight given by
-    get_edge(u,v) one if there is an edge u-v and zero otherwise.
-
-    Graphs with multi-edges are not handled.
+    Multiple edges and edge data are both ignored for MultiGraph/MultiDiGraph.
 
     """
     try:
@@ -256,36 +269,32 @@ def to_numpy_matrix(G,nodelist=None):
         raise ImportError, \
               "Import Error: not able to import numpy: http://numpy.scipy.org "
 
-    if hasattr(G,"multiedges"):
-        if G.multiedges:
-            raise TypeError, \
-                  "Conversion to numpy_matrix not allowed with for graphs with multiedges."
 
     if nodelist is None:
         nodelist=G.nodes()
     nlen=len(nodelist)    
     index=dict(zip(nodelist,range(nlen)))# dict mapping vertex name to position
     A = numpy.asmatrix(numpy.zeros((nlen,nlen)))
-    directed=G.is_directed()
-    for e in G.edges_iter(nodelist):
-        u=e[0]
-        v=e[1]
-        if len(e)==2:
-            d=1
-        else:
-            d=e[2]
-            if d is None: d=1 # None would be a nan in numpy, use 1
-        if u not in nodelist or v not in nodelist:
-            continue
-        A[index[u],index[v]]=d
-        if not directed:
-            A[index[v],index[u]]=d
+    if G.weighted and not G.multigraph:
+        for n,nbrdict in G.adjacency_iter():
+            if n in index:
+                for nbr,data in nbrdict.iteritems():
+                    if nbr in index:
+                        A[index[n],index[nbr]]=data
+    else: # ignore edge data
+        for n,nbrdict in G.adjacency_iter():
+            if n in index:
+                for nbr in nbrdict:
+                    if nbr in index:
+                        A[index[n],index[nbr]]=1
     return A            
 
 def from_numpy_matrix(A,create_using=None):
     """Return networkx graph G from numpy matrix adjacency list. 
 
-    >>> G=from_numpy_matrix(A)
+    >>> import numpy
+    >>> A=numpy.matrix([[1,1],[2,1]])
+    >>> G=nx.from_numpy_matrix(A)
 
     """
     # This should never fail if you have created a numpy matrix with numpy...  
@@ -307,17 +316,7 @@ def from_numpy_matrix(A,create_using=None):
 
     # get a list of edges
     x,y=numpy.asarray(A).nonzero()         
-    # is this a XGraph or XDiGraph?
-    # FIXME
-    # This is a bad way to check for whether edge data exists...
-    # If someone ever creates a graph class with edge data and
-    # without an allow_multiedges method, it won't work.
-    if hasattr(G,'allow_multiedges'):
-        for (u,v) in zip(x,y):        
-            G.add_edge(u,v,A[u,v])
-    else:
-        for (u,v) in zip(x,y):        
-            G.add_edge(u,v)                
+    G.add_edges_from( ((u,v,A[u,v]) for (u,v) in zip(x,y)) )
     return G
 
 
@@ -327,23 +326,17 @@ def to_scipy_sparse_matrix(G,nodelist=None):
     Uses lil_matrix format.  To convert to other formats see
     scipy.sparse documentation.
 
-    If nodelist is defined return adjacency matrix with nodes in nodelist
-    in the order specified.  If not the ordering is whatever order
-    the method G.nodes() produces.
+    If nodelist is defined return adjacency matrix with nodes 
+    in the order specified by nodelist.  Otherwise the order
+    is produced by G.nodes().
 
-    For Graph/DiGraph types which have no edge data 
-    The value of the entry A[u,v] is one if there is an edge u-v
-    and zero otherwise.
+    If G.weighted is True and G is not a multigraph, the edgedata
+    is assumed to be numeric and becomes the value of A[u.v].
+    Otherwise A[u,v] is one if an edge u-v exists and zero otherwise.
 
-    For XGraph/XDiGraph the edge data is assumed to be a weight and be
-    able to be converted to a valid numpy type (e.g. an int or a
-    float).  The value of the entry A[u,v] is the weight given by
-    get_edge(u,v) one if there is an edge u-v and zero otherwise.
-
-    Graphs with multi-edges are not handled.
-
-    >>> A=scipy_sparse_matrix(G)
-    >>> A.tocsr() # convert to compressed row storage
+    >>> G=nx.path_graph(4)
+    >>> A=nx.to_scipy_sparse_matrix(G)
+    >>> C=A.tocsr() # convert to compressed row storage
 
     """
     try:
@@ -353,29 +346,24 @@ def to_scipy_sparse_matrix(G,nodelist=None):
               """Import Error: not able to import scipy sparse:
               see http://scipy.org""" 
 
-    if hasattr(G,"multiedges"):
-        if G.multiedges:
-            raise TypeError, \
-                  "Conversion to scipy_sparse_matrix not allowed with for graphs with multiedges."
 
     if nodelist is None:
         nodelist=G.nodes()
     nlen=len(nodelist)    
     index=dict(zip(nodelist,range(nlen)))# dict mapping vertex name to position
     A = sparse.lil_matrix((nlen,nlen))
-    for e in G.edges_iter(nodelist):
-        u=e[0]
-        v=e[1]
-        if len(e)==2:
-            d=1
-        else:
-            d=e[2]
-            if d is None: d=1 # None would be a nan, use 1 
-        if u not in nodelist or v not in nodelist:
-            continue
-        A[index[u],index[v]]=d
-        if not G.is_directed():
-            A[index[v],index[u]]=d
+    if G.weighted and not G.multigraph:
+        for n,nbrdict in G.adjacency_iter():
+            if n in index:
+                for nbr,data in nbrdict.iteritems():
+                    if nbr in index:
+                        A[index[n],index[nbr]]=data
+    else: # ignore edge data
+        for n,nbrdict in G.adjacency_iter():
+            if n in index:
+                for nbr in nbrdict:
+                    if nbr in index:
+                        A[index[n],index[nbr]]=1
     return A            
 
 
@@ -383,9 +371,12 @@ def from_scipy_sparse_matrix(A,create_using=None):
     """Return networkx graph G from scipy scipy sparse matrix
     adjacency list. 
 
-    >>> G=from_scipy_sparse_matrix(A)
+    >>> import scipy.sparse
+    >>> A=scipy.sparse.eye(2,2,1)
+    >>> G=nx.from_scipy_sparse_matrix(A)
 
     """
+
     G=_prep_create_using(create_using)
 
     # is this a XGraph or XDiGraph?
@@ -398,8 +389,8 @@ def from_scipy_sparse_matrix(A,create_using=None):
     else:
         xgraph=False
 
-    # convert everything to coo - not the most efficient        
-    AA=A.tocoo()
+    # convert all formats to lil - not the most efficient way       
+    AA=A.tolil()
     nx,ny=AA.shape
 
     if nx!=ny:
@@ -408,13 +399,14 @@ def from_scipy_sparse_matrix(A,create_using=None):
 
 
     G.add_nodes_from(range(nx)) # make sure we get isolated nodes
-    for i in range(AA.nnz):
-        e=AA.rowcol(i)
-        if xgraph:
-            e=(e[0],e[1],AA.getdata(i))
-        G.add_edge(e)
-    return G
 
+    for i,row in enumerate(AA.rows):
+        for pos,j in enumerate(row):
+           if xgraph:
+               G.add_edge(i,j,AA.data[i][pos])
+           else:
+               G.add_edge(i,j)
+    return G
 
 def _test_suite():
     import doctest
