@@ -1,19 +1,22 @@
 """
+****
+YAML
+****
+
 Read and write NetworkX graphs in YAML format.
 See http://www.yaml.org for documentation.
 
 """
 __author__ = """Aric Hagberg (hagberg@lanl.gov)"""
-__date__ = """"""
-__credits__ = """"""
-__revision__ = "$$"
-#    Copyright (C) 2004-2007 by 
+#    Copyright (C) 2004-2008 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    Distributed under the terms of the GNU Lesser General Public License
 #    http://www.gnu.org/copyleft/lesser.html
 
+__all__ = ['read_yaml', 'write_yaml']
+
 import cPickle 
 import codecs
 import locale
@@ -55,22 +58,3 @@ def read_yaml(path):
     return yaml.load(fh)
 
 
-
-def _test_suite():
-    import doctest
-    import yaml
-    suite = doctest.DocFileSuite('tests/readwrite/nx_yaml.txt',package='networkx')
-    return suite
-
-if __name__ == "__main__":
-    import os 
-    import sys
-    import unittest
-    if sys.version_info[:2] < (2, 4):
-        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
-        sys.exit(-1)
-    # directory of networkx package (relative to this)
-    nxbase=sys.path[0]+os.sep+os.pardir
-    sys.path.insert(0,nxbase) # prepend to search path
-    unittest.TextTestRunner().run(_test_suite())
-    
