@@ -7,10 +7,34 @@
 #    Distributed under the terms of the GNU Lesser General Public License
 #    http://www.gnu.org/copyleft/lesser.html
 
-__author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult (dschult@colgate.edu)"""
+
+import os
+import re
+
+
+def get_svn_revision():
+    #import networkx
+    rev = None
+    path ="."
+    entries_path = '%s/.svn/entries' % path
+    if os.path.exists(entries_path):
+        entries = open(entries_path, 'r').read()
+        # Versions >= 7 of the entries file are flat text.  The first line is
+        # the version number. The next set of digits after 'dir' is the revision.
+        if re.match('(\d+)', entries):
+            rev_match = re.search('\d+\s+dir\s+(\d+)', entries)
+            if rev_match:
+                rev = rev_match.groups()[0]
+    if rev:
+        return 'dev%s' % rev
+    return None
+
 
 name = 'networkx'
-version = '0.36'
+version = '0.99'
+revision = None
+if revision is not None:
+    version+=".%s"%revision
 
 description = "Python package for creating and manipulating graphs and networks"
 
@@ -26,7 +50,7 @@ authors = {'Hagberg' : ('Aric Hagberg','hagberg@lanl.gov'),
            'Swart' : ('Pieter Swart','swart@lanl.gov')
            }
 url = 'http://networkx.lanl.gov/'
-download_url="http://networkx.lanl.gov/download"
+download_url="http://networkx.lanl.gov/download/networkx"
 platforms = ['Linux','Mac OSX','Windows XP/2000/NT']
 keywords = ['Networks', 'Graph Theory', 'Mathematics', 'network', 'graph', 'discrete mathematics', 'math']
 classifiers = [
