@@ -10,13 +10,13 @@ you will get a python egg.
 
 Use
 
->>> python setup_egg.py test
+>>> python setup_egg.py nosetests
 
 to run the tests.
 
 
 """
-from setuptools import setup, Extension
+from setuptools import setup
 from setup import *
 
 if __name__ == "__main__":
@@ -37,8 +37,11 @@ if __name__ == "__main__":
       data_files       = data,
       classifiers      = classifiers,
       package_data     = package_data,
-      include_package_data = True,
-      test_suite       = "networkx.tests.test.all", 
+#      include_package_data = True,
+      install_requires=['setuptools'],
+      test_suite       = 'nose.collector', 
+      tests_require    = ['nose >= 0.10.1','networkx-nose-plugin>=0.1'] ,
+      zip_safe = True
       )
 
 
